local title = "LazyCurve"

local coreFrame = CreateFrame("Frame");
local core = LibStub("AceAddon-3.0"):NewAddon(coreFrame, title, "AceHook-3.0") 
LazyCurve = core

local gsub = gsub
local table = table
local pairs = pairs
local string_find = string.find
local strupper = strupper
local GetAchievementLink = GetAchievementLink

local defaultCurve = 12110 --latest curve
local defaultEdge = 12111 -- latest edge
local curveTable

--[[
creating ordered_table:
tablevariable = ordered_table.new{
    key1, value1, (!!so not key=val; but key, val!!)
    key2, value2,
}
--]]
local masterTable = {}

masterTable.raids = {
    "ABT",
    "TOS",
    "NH",
    "TOV",
    "EN",
}

masterTable.finalBoss = {
    ABT = "argus",
    TOS = "kiljaeden",
    NH = "guldan",
    TOV = "helya",
    EN = "xavius"
}

masterTable.ABT = { --first 3 are nm, curve and edge; rest is mythic
    12266, --nm/lfr argus kill
    12110, --curve
    12111, --edge
    11992, --garothi worldbreaker
    11993, --hounds
    11994, --high command
    11995, --portal keeper
    11996, --eonar
    11997, --imonar
    11998, --kin'garoth
    11999, -- varimathras
    12000, --coven
    12001, --aggramar
    12002, --argus
    12111, --edge
}

masterTable.TOS = { --first 3 are nm, curve and edge; rest is mythic
    11790, --nm/lfr KJ kill
    11874, --curve
    11875, --edge
    11767, --goroth
    11774, --demonic inq
    11775, --harja
    11776, --mistress
    11777, --sisters
    11778, --host
    11779, --maiden
    11780, --avatar
    11781, --KJ
    11875, -- edge
}

masterTable.NH = { --first 3 are nm, curve and edge; rest is mythic
    10839, --nm/lfr guldan
    11195, -- curve
    11192, --edge
    10840, --skorpyron
    10842, --chronomatic anomaly
    10843, --trilliax
    10844, --spellblade aluriel
    10845, --star augur
    10846, --botanist
    10847, --tichondrius
    10848, --krosus
    10849, --elisande
    10850, --guldan
    11192, --edge
}

masterTable.TOV = { --first 3 are nm, curve and edge; rest is mythic
    11394, --nm/lfr helya
    11581, -- curve
    11580, --edge
    11396, --odig
    11397, --guarm
    11398, --helya
    11580, --edge
}

masterTable.EN = { --first 3 are nm, curve and edge; rest is mythic
    10820, --nm/lfr xavius
    11194, --curve
    11191, --edge
    10821, --nythendra
    10822, --elerethe renferal
    10823, --il'gynoth
    10824, --ursoc
    10825, --dragons
    10826, --cenarius
    10827, --xavius
    11191, --edge
}


function core:getHighestMythicAchievement(raidname)
    local retID = defaultEdge
    if(masterTable[raidname] ~= nil) then
        for k, chievoID in pairs(masterTable[raidname]) do
            local _, chievoName, _, completed, _ = GetAchievementInfo(chievoID)
            if(completed and k > 3) then
                retID = chievoID
            end
        end
    end
    return retID
end

function core:getCurveTable(filter)
    t = ordered_table.new{}
    if(filter == "nofilter" or filter == "raids" or filter == "all") then
        for k,raidname in pairs(masterTable.raids) do
            local _, chievoName, _, completed, _ = GetAchievementInfo(masterTable[raidname][2])
            if(not completed or filter == "nofilter") then -- don't show nm/lfr chievo unless using a KEYWORD or you don't have curve
                t[raidname.."normal"] = masterTable[raidname][1]
            end
            t[raidname.."curve"] = masterTable[raidname][2]
            t[raidname.."edge"] = masterTable[raidname][3]
            t[raidname.."mythic"] = core.getHighestMythicAchievement(self,raidname)
            
            --nickname
            if(filter == "nofilter") then
                t[masterTable.finalBoss[raidname].."normal"] = masterTable[raidname][1]
                t[masterTable.finalBoss[raidname].."curve"] = masterTable[raidname][2]
                t[masterTable.finalBoss[raidname].."edge"] = masterTable[raidname][3]
                t[masterTable.finalBoss[raidname].."mythic"] = core.getHighestMythicAchievement(self,raidname)
            end
        end
        if(filter == "nofilter") then
            t["curve"] = defaultCurve
            t["edge"] = defaultEdge
        end
    end
    
    if(filter == "nofilter" or filter == "dungeons" or filter == "all") then
        t["KEY15"] = 11162
        t["KEY10"] = 11185
        t["KEY5"] = 11184
        t["KEY2"] =11183
    end
    
    return t
end

local raidActivityTable = {
    [483] = "ABT",
    [482] = "ABT",
    
    [492] = "TOS",
    [478] = "TOS",
    [479] = "TOS",
    
    [481] = "NH",
    [416] = "NH",
    [415] = "NH",
    
    [457] = "TOV",
    [480] = "TOV",
    [456] = "TOV",
    
    [468] = "EN",
    [414] = "EN",
    [413] = "EN",
}

local dungeonCatID = 2
local raidCatID = 3


-- Ace3 Functions
function core:OnInitialize()
    self:RawHook("SendChatMessage", true)
    self:RawHook("LFGListUtil_GetSearchEntryMenu",true)
    curveTable = core.getCurveTable(self, "nofilter")
end

-- update tools
local function tprint( data, level )
	level = level or 0
	local ident=strrep('    ', level)
	if level>5 then return end

	if type(data)~='table' then print(tostring(data)) return end;

	for index,value in pairs(data) do repeat
		if type(value)~='table' then
			print( ident .. '['..index..'] = ' .. tostring(value) .. ' (' .. type(value) .. ')' );
			break;
		end
		print( ident .. '['..index..'] = {')
        tprint(value, level+1)
        print( ident .. '}' );
	until true end
end

function core:getAllCategoryInfo() --/run LazyCurve.getAllCategoryInfo()
    local tempTable ={}; 
    for i=1,10 do 
        tempTable[i] = C_LFGList.GetCategoryInfo(i) 
    end;
    tprint(tempTable)
end

function core:getAllActivityInfo(dungeonCatID, raidCatID) --/run LazyCurve.getAllActivityInfo(_, dungeonCatID, raidCatID)
    local dungeonActivities = C_LFGList.GetAvailableActivities(dungeonCatID)
    for key, val in pairs(dungeonActivities) do
        fullName, shortName, catID, _, _, _, minLevel, _, displayType, _ = C_LFGList.GetActivityInfo(val)
        if minLevel >= 110 then
            print("activityID " .. val .. ":" .. fullName .. "(" .. shortName .. ") minlvl:" .. minLevel .. "; displaytype:" .. displayType)
        end
    end
    print("-------------")
    local raidActivities = C_LFGList.GetAvailableActivities(raidCatID)
    for key, val in pairs(raidActivities) do
        fullName, shortName, catID, _, _, _, minLevel, _, displayType, _ = C_LFGList.GetActivityInfo(val)
        if minLevel >= 110 then 
            print("activityID " .. val .. ":" .. fullName .. "(" .. shortName .. ") minlvl:" .. minLevel .. "; displaytype:" .. displayType)
        end
    end
end
-- end update tools

------------------------------------------------------------------
function core:SendChatMessage(...)
    -- Our hook to SendChatMessage. If msg contains a tracked keyword, we insert the achievement.
    local msg, chatType, language, channel = ...;
    
    for keyword, chievoID in ordered_table.pairs(curveTable) do
        keyword = strupper(keyword)
        if string_find(msg, keyword) then --keyword found
            local found, _ = string_find(msg, keyword)
            while(found ~= nil) do --repeat untill all keywords are replaced
                msg, _ = gsub(msg, keyword, GetAchievementLink(chievoID))
                found, _ = string_find(msg, keyword)
            end
        end
    end
    core.hooks.SendChatMessage(msg, chatType, language, channel);
end

function core:SendCurve(name, curve)
    for keyword, chievoID in pairs(curveTable) do
        if curve == chievoID then
            core.SendChatMessage(self, keyword, "WHISPER", nil, name)
            return
        end
    end
end

function core:getFilteredCurveTable(activityID, override)
    fullName, difficulty, catID, _, _, _, minLevel, _ = C_LFGList.GetActivityInfo(activityID)
    local supportedActivity = false
    if(catID == dungeonCatID) then --dungeon chievos
        if(minLevel == 110) then
            supportedActivity = true
            filteredCurveTable = core.getCurveTable(self,"dungeons")
        end
    elseif(catID == raidCatID) then --raid chievos
        supportedActivity = true
        if(raidActivityTable[activityID] ~= nil) then
            filteredCurveTable = ordered_table.new{
                raidActivityTable[activityID].."normal", curveTable[raidActivityTable[activityID].."normal"],
                raidActivityTable[activityID].."curve", curveTable[raidActivityTable[activityID].."curve"],
                raidActivityTable[activityID].."edge", curveTable[raidActivityTable[activityID].."edge"],
                raidActivityTable[activityID].."mythic", curveTable[raidActivityTable[activityID].."mythic"]
            }
            if(curveTable[raidActivityTable[activityID].."curve"] ~= curveTable.curve)then
                filteredCurveTable.curve = curveTable.curve
            end
            if(curveTable[raidActivityTable[activityID].."edge"] ~= curveTable.edge)then
                filteredCurveTable.edge = curveTable.edge
            end
        else
            filteredCurveTable = core.getCurveTable(self,"raids")
        end
    end
    
    if(not supportedActivity) then -- all chievos
        filteredCurveTable = core.getCurveTable(self,"all")
    end
    return filteredCurveTable
end

function core:getExtraMenuList(activityID, leaderName)
    local tempMenuList = {}
    
    filteredCurveTable = core.getFilteredCurveTable(self, activityID)
    for keyword, chievoID in ordered_table.pairs(filteredCurveTable) do
        local _, chievoName, _, completed, _ = GetAchievementInfo(chievoID)
        if(completed) then
            table.insert(tempMenuList, {
                text =chievoName,
                func = function(_, name) core.SendCurve(self, name, chievoID); end,
                notCheckable = true,
                arg1 = leaderName, --Leader name goes here
                disabled = not leaderName, --Disabled if we don't have a leader name yet or you haven't applied
            })
            if(string_find(keyword, "KEY")) then
                break
            end
        end
    end
    if(table.getn(tempMenuList)>1) then
        return { 
                text = "Whisper Achievement to Leader",
                hasArrow = true,
                notCheckable = true,
                disabled = not leaderName,
                menuList = tempMenuList,
        };
    end
    if(table.getn(tempMenuList) == 1) then
        tempMenuList[1].text = "Whisper " .. tempMenuList[1].text .. " Achievement to Leader"
        return tempMenuList[1]
    end
end

function core:LFGListUtil_GetSearchEntryMenu(resultID)
    local _, activityID, _, _, _, _, _, _, _, _, _, _, leaderName = C_LFGList.GetSearchResultInfo(resultID);
    local LFG_LIST_SEARCH_ENTRY_MENU = nil
    LFG_LIST_SEARCH_ENTRY_MENU = core.hooks.LFGListUtil_GetSearchEntryMenu(resultID)
    LFG_LIST_SEARCH_ENTRY_MENU[2].tooltipTitle = nil
    LFG_LIST_SEARCH_ENTRY_MENU[2].tooltipText = nil
    
    local lastItem = table.remove(LFG_LIST_SEARCH_ENTRY_MENU) -- the cancel button
    local checking = true
    local temp
    while (checking) do
        temp = table.remove(LFG_LIST_SEARCH_ENTRY_MENU)
        if(not string_find(temp.text, "Achievement to Leader")) then
            table.insert(LFG_LIST_SEARCH_ENTRY_MENU, temp)
            checking = false
        end
    end
    
    table.insert(LFG_LIST_SEARCH_ENTRY_MENU, core.getExtraMenuList(self, activityID, leaderName))
    
    table.insert(LFG_LIST_SEARCH_ENTRY_MENU, lastItem) -- put the cancel button back in it's place
    return LFG_LIST_SEARCH_ENTRY_MENU; --and all is well that ends well :)
end



